/* Gpx Viewer 
 * Copyright (C) 2009-2009 Qball Cow <qball@sarine.nl>
 * Project homepage: http://blog.sarine.nl/ 

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <gio/gio.h>
#include <libxml/tree.h>
#include <libxml/xmlreader.h>
#include <libxml/xmlIO.h>
#include <gobject/gvaluecollector.h>


#define GPX_TYPE_POINT (gpx_point_get_type ())
#define GPX_POINT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_POINT, GpxPoint))
#define GPX_POINT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_POINT, GpxPointClass))
#define GPX_IS_POINT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_POINT))
#define GPX_IS_POINT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_POINT))
#define GPX_POINT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_POINT, GpxPointClass))

typedef struct _GpxPoint GpxPoint;
typedef struct _GpxPointClass GpxPointClass;
typedef struct _GpxPointPrivate GpxPointPrivate;
typedef struct _GpxParamSpecPoint GpxParamSpecPoint;

#define GPX_TYPE_TRACK (gpx_track_get_type ())
#define GPX_TRACK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_TRACK, GpxTrack))
#define GPX_TRACK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_TRACK, GpxTrackClass))
#define GPX_IS_TRACK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_TRACK))
#define GPX_IS_TRACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_TRACK))
#define GPX_TRACK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_TRACK, GpxTrackClass))

typedef struct _GpxTrack GpxTrack;
typedef struct _GpxTrackClass GpxTrackClass;
typedef struct _GpxTrackPrivate GpxTrackPrivate;

#define GPX_TYPE_FILE (gpx_file_get_type ())
#define GPX_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_FILE, GpxFile))
#define GPX_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_FILE, GpxFileClass))
#define GPX_IS_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_FILE))
#define GPX_IS_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_FILE))
#define GPX_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_FILE, GpxFileClass))

typedef struct _GpxFile GpxFile;
typedef struct _GpxFileClass GpxFileClass;
typedef struct _GpxFilePrivate GpxFilePrivate;

/**
 * Represents a point in the track or a waypoint. 
 */
struct _GpxPoint {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GpxPointPrivate * priv;
	double lat;
	double lon;
	double lat_dec;
	double lon_dec;
	double distance;
	double elevation;
	char* time;
	double speed;
};

struct _GpxPointClass {
	GTypeClass parent_class;
	void (*finalize) (GpxPoint *self);
};

struct _GpxPointPrivate {
	char* _name;
	time_t utime;
};

struct _GpxParamSpecPoint {
	GParamSpec parent_instance;
};

/**
 * This class represents a Track in a gpx file.
 * The tracks contains the points connecting everything together. 
 * Info like total distance, average speed, moving speed/time etc are available.
 */
struct _GpxTrack {
	GObject parent_instance;
	GpxTrackPrivate * priv;
	double total_distance;
	double max_speed;
	GList* points;
	GpxPoint* top;
	GpxPoint* bottom;
};

struct _GpxTrackClass {
	GObjectClass parent_class;
};

struct _GpxTrackPrivate {
	char* _name;
	GpxPoint* last;
};

/**
 * This is the top level class representing the gpx file it self.
 * This contains a list of tracks and waypoings.
 */
struct _GpxFile {
	GObject parent_instance;
	GpxFilePrivate * priv;
	char* filename;
	GList* tracks;
	GList* waypoints;
	GList* routes;
};

struct _GpxFileClass {
	GObjectClass parent_class;
};

struct _GpxFilePrivate {
	GFile* file;
	GFileInputStream* stream;
};


static gpointer gpx_point_parent_class = NULL;
static gpointer gpx_track_parent_class = NULL;
static gpointer gpx_file_parent_class = NULL;

gpointer gpx_point_ref (gpointer instance);
void gpx_point_unref (gpointer instance);
GParamSpec* gpx_param_spec_point (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gpx_value_set_point (GValue* value, gpointer v_object);
gpointer gpx_value_get_point (const GValue* value);
GType gpx_point_get_type (void);
#define GPX_POINT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPX_TYPE_POINT, GpxPointPrivate))
enum  {
	GPX_POINT_DUMMY_PROPERTY
};
void gpx_point_set_position (GpxPoint* self, double lat, double lon);
time_t gpx_point_get_time (GpxPoint* self);
GpxPoint* gpx_point_new (void);
GpxPoint* gpx_point_construct (GType object_type);
const char* gpx_point_get_name (GpxPoint* self);
void gpx_point_set_name (GpxPoint* self, const char* value);
static void gpx_point_finalize (GpxPoint* obj);
GType gpx_track_get_type (void);
#define GPX_TRACK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPX_TYPE_TRACK, GpxTrackPrivate))
enum  {
	GPX_TRACK_DUMMY_PROPERTY,
	GPX_TRACK_NAME
};
static void _g_list_free_gpx_point_unref (GList* self);
static double gpx_track_calculate_distance (GpxTrack* self, GpxPoint* a, GpxPoint* b);
double gpx_track_calculate_point_to_point_speed (GpxTrack* self, GpxPoint* a, GpxPoint* b);
void gpx_track_add_point (GpxTrack* self, GpxPoint* point);
double gpx_track_get_track_average (GpxTrack* self);
time_t gpx_track_get_total_time (GpxTrack* self);
double gpx_track_calculate_moving_average (GpxTrack* self, GpxPoint* start, GpxPoint* stop, time_t* moving_time);
GpxTrack* gpx_track_new (void);
GpxTrack* gpx_track_construct (GType object_type);
const char* gpx_track_get_name (GpxTrack* self);
void gpx_track_set_name (GpxTrack* self, const char* value);
static void gpx_track_finalize (GObject* obj);
static void gpx_track_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void gpx_track_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType gpx_file_get_type (void);
#define GPX_FILE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPX_TYPE_FILE, GpxFilePrivate))
enum  {
	GPX_FILE_DUMMY_PROPERTY
};
static void _g_list_free_g_object_unref (GList* self);
static void gpx_file_parse_track (GpxFile* self, xmlNode* node);
static void gpx_file_parse_waypoint (GpxFile* self, xmlNode* node);
static void gpx_file_parse_route (GpxFile* self, xmlNode* node);
static gint gpx_file_read_file (GpxFile* self, gchar* buffer, int buffer_length1);
static gint gpx_file_close_file (GpxFile* self);
GpxFile* gpx_file_new (const char* filename);
GpxFile* gpx_file_construct (GType object_type, const char* filename);
static void gpx_file_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



/* Sets the poistion in degrees, automagically calculates radians */
void gpx_point_set_position (GpxPoint* self, double lat, double lon) {
	g_return_if_fail (self != NULL);
	self->lat_dec = lat;
	self->lon_dec = lon;
	self->lat = ((2 * G_PI) * lat) / 360.0;
	self->lon = ((2 * G_PI) * lon) / 360.0;
}


/* Get the unix time. (calculated on first request, then cached ) */
time_t gpx_point_get_time (GpxPoint* self) {
	time_t result;
	struct tm _tmp0_ = {0};
	struct tm ta;
	if (self->time == NULL) {
		result = 0;
		return result;
	}
	if (self->priv->utime > 0) {
		result = self->priv->utime;
		return result;
	}
	ta = (memset (&_tmp0_, 0, sizeof (struct tm)), _tmp0_);
	strptime (self->time, "%FT%T%z", &ta);
	self->priv->utime = mktime (&ta);
	result = self->priv->utime;
	return result;
}


/**
 * Represents a point in the track or a waypoint. 
 */
GpxPoint* gpx_point_construct (GType object_type) {
	GpxPoint* self;
	self = (GpxPoint*) g_type_create_instance (object_type);
	return self;
}


GpxPoint* gpx_point_new (void) {
	return gpx_point_construct (GPX_TYPE_POINT);
}


const char* gpx_point_get_name (GpxPoint* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_name;
	return result;
}


void gpx_point_set_name (GpxPoint* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_name = (g_free (self->priv->_name), NULL), _tmp1_);
}


static void gpx_value_point_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void gpx_value_point_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		gpx_point_unref (value->data[0].v_pointer);
	}
}


static void gpx_value_point_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = gpx_point_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer gpx_value_point_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* gpx_value_point_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		GpxPoint* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = gpx_point_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* gpx_value_point_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	GpxPoint** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = gpx_point_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* gpx_param_spec_point (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	GpxParamSpecPoint* spec;
	g_return_val_if_fail (g_type_is_a (object_type, GPX_TYPE_POINT), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer gpx_value_get_point (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GPX_TYPE_POINT), NULL);
	return value->data[0].v_pointer;
}


void gpx_value_set_point (GValue* value, gpointer v_object) {
	GpxPoint* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GPX_TYPE_POINT));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, GPX_TYPE_POINT));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		gpx_point_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		gpx_point_unref (old);
	}
}


static void gpx_point_class_init (GpxPointClass * klass) {
	gpx_point_parent_class = g_type_class_peek_parent (klass);
	GPX_POINT_CLASS (klass)->finalize = gpx_point_finalize;
	g_type_class_add_private (klass, sizeof (GpxPointPrivate));
}


static void gpx_point_instance_init (GpxPoint * self) {
	self->priv = GPX_POINT_GET_PRIVATE (self);
	self->priv->_name = NULL;
	self->lat = (double) 1000;
	self->lon = (double) 1000;
	self->lat_dec = (double) 1000;
	self->lon_dec = (double) 1000;
	self->distance = (double) 0;
	self->speed = (double) 0;
	self->priv->utime = 0;
	self->ref_count = 1;
}


static void gpx_point_finalize (GpxPoint* obj) {
	GpxPoint * self;
	self = GPX_POINT (obj);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	self->time = (g_free (self->time), NULL);
}


GType gpx_point_get_type (void) {
	static GType gpx_point_type_id = 0;
	if (gpx_point_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { gpx_value_point_init, gpx_value_point_free_value, gpx_value_point_copy_value, gpx_value_point_peek_pointer, "p", gpx_value_point_collect_value, "p", gpx_value_point_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (GpxPointClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gpx_point_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GpxPoint), 0, (GInstanceInitFunc) gpx_point_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		gpx_point_type_id = g_type_register_fundamental (g_type_fundamental_next (), "GpxPoint", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return gpx_point_type_id;
}


gpointer gpx_point_ref (gpointer instance) {
	GpxPoint* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void gpx_point_unref (gpointer instance) {
	GpxPoint* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		GPX_POINT_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _g_list_free_gpx_point_unref (GList* self) {
	g_list_foreach (self, (GFunc) gpx_point_unref, NULL);
	g_list_free (self);
}


void gpx_track_add_point (GpxTrack* self, GpxPoint* point) {
	GpxPoint* _tmp13_;
	GpxPoint* _tmp15_;
	GpxPoint* _tmp14_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (point != NULL);
	if (self->priv->last != NULL) {
		double distance;
		gboolean _tmp0_;
		gboolean _tmp1_;
		gboolean _tmp2_;
		gboolean _tmp4_;
		gboolean _tmp5_;
		gboolean _tmp7_;
		gboolean _tmp8_;
		gboolean _tmp10_;
		gboolean _tmp11_;
		distance = gpx_track_calculate_distance (self, self->priv->last, point);
		self->total_distance = self->total_distance + distance;
		point->distance = self->total_distance;
		_tmp0_ = FALSE;
		if (self->priv->last->time != NULL) {
			_tmp0_ = point->time != NULL;
		} else {
			_tmp0_ = FALSE;
		}
		if (_tmp0_) {
			double avg;
			point->speed = gpx_track_calculate_point_to_point_speed (self, self->priv->last, point);
			if ((gpx_track_calculate_point_to_point_speed (self, (GpxPoint*) g_list_first (self->points)->data, point) * 4) < point->speed) {
				point->speed = self->priv->last->speed;
			}
			avg = point->speed;
			if (avg > self->max_speed) {
				self->max_speed = avg;
			}
		}
		_tmp1_ = FALSE;
		_tmp2_ = FALSE;
		if (self->top == NULL) {
			_tmp2_ = TRUE;
		} else {
			_tmp2_ = self->top->lat_dec == 1000;
		}
		if (_tmp2_) {
			_tmp1_ = TRUE;
		} else {
			_tmp1_ = self->top->lat_dec > point->lat_dec;
		}
		/* Update the 2 bounding box points */
		if (_tmp1_) {
			if (self->top == NULL) {
				GpxPoint* _tmp3_;
				_tmp3_ = NULL;
				self->top = (_tmp3_ = gpx_point_new (), (self->top == NULL) ? NULL : (self->top = (gpx_point_unref (self->top), NULL)), _tmp3_);
			}
			self->top->lat_dec = point->lat_dec;
		}
		_tmp4_ = FALSE;
		_tmp5_ = FALSE;
		if (self->top == NULL) {
			_tmp5_ = TRUE;
		} else {
			_tmp5_ = self->top->lon_dec == 1000;
		}
		if (_tmp5_) {
			_tmp4_ = TRUE;
		} else {
			_tmp4_ = self->top->lon_dec > point->lon_dec;
		}
		if (_tmp4_) {
			if (self->top == NULL) {
				GpxPoint* _tmp6_;
				_tmp6_ = NULL;
				self->top = (_tmp6_ = gpx_point_new (), (self->top == NULL) ? NULL : (self->top = (gpx_point_unref (self->top), NULL)), _tmp6_);
			}
			self->top->lon_dec = point->lon_dec;
		}
		_tmp7_ = FALSE;
		_tmp8_ = FALSE;
		if (self->bottom == NULL) {
			_tmp8_ = TRUE;
		} else {
			_tmp8_ = self->bottom->lat_dec == 1000;
		}
		if (_tmp8_) {
			_tmp7_ = TRUE;
		} else {
			_tmp7_ = self->bottom->lat_dec < point->lat_dec;
		}
		if (_tmp7_) {
			if (self->bottom == NULL) {
				GpxPoint* _tmp9_;
				_tmp9_ = NULL;
				self->bottom = (_tmp9_ = gpx_point_new (), (self->bottom == NULL) ? NULL : (self->bottom = (gpx_point_unref (self->bottom), NULL)), _tmp9_);
			}
			self->bottom->lat_dec = point->lat_dec;
		}
		_tmp10_ = FALSE;
		_tmp11_ = FALSE;
		if (self->bottom == NULL) {
			_tmp11_ = TRUE;
		} else {
			_tmp11_ = self->bottom->lon_dec == 1000;
		}
		if (_tmp11_) {
			_tmp10_ = TRUE;
		} else {
			_tmp10_ = self->bottom->lon_dec < point->lon_dec;
		}
		if (_tmp10_) {
			if (self->bottom == NULL) {
				GpxPoint* _tmp12_;
				_tmp12_ = NULL;
				self->bottom = (_tmp12_ = gpx_point_new (), (self->bottom == NULL) ? NULL : (self->bottom = (gpx_point_unref (self->bottom), NULL)), _tmp12_);
			}
			self->bottom->lon_dec = point->lon_dec;
		}
	}
	_tmp13_ = NULL;
	self->points = g_list_append (self->points, (_tmp13_ = point, (_tmp13_ == NULL) ? NULL : gpx_point_ref (_tmp13_)));
	_tmp15_ = NULL;
	_tmp14_ = NULL;
	self->priv->last = (_tmp15_ = (_tmp14_ = point, (_tmp14_ == NULL) ? NULL : gpx_point_ref (_tmp14_)), (self->priv->last == NULL) ? NULL : (self->priv->last = (gpx_point_unref (self->priv->last), NULL)), _tmp15_);
}


/* Private api 
*
 * Calculate the speed of the full track
 */
double gpx_track_get_track_average (GpxTrack* self) {
	double result;
	GList* first;
	GList* last;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, 0.0);
	first = g_list_first (self->points);
	last = g_list_last (self->points);
	_tmp0_ = FALSE;
	if (first != NULL) {
		_tmp0_ = last != NULL;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = gpx_track_calculate_point_to_point_speed (self, (GpxPoint*) first->data, (GpxPoint*) last->data);
		return result;
	}
	result = (double) 0;
	return result;
}


/**
 * Calculate the average speed between Point a and Point b on the track
 */
double gpx_track_calculate_point_to_point_speed (GpxTrack* self, GpxPoint* a, GpxPoint* b) {
	double result;
	double dist;
	gboolean _tmp0_;
	time_t ta;
	time_t tb;
	g_return_val_if_fail (self != NULL, 0.0);
	g_return_val_if_fail (a != NULL, 0.0);
	g_return_val_if_fail (b != NULL, 0.0);
	dist = b->distance - a->distance;
	_tmp0_ = FALSE;
	if (a->time == NULL) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = b->time == NULL;
	}
	if (_tmp0_) {
		result = (double) 0;
		return result;
	}
	ta = gpx_point_get_time (a);
	tb = gpx_point_get_time (b);
	if ((tb - ta) == 0) {
		result = (double) 0;
		return result;
	}
	result = dist / ((tb - ta) / (60.0 * 60.0));
	return result;
}


time_t gpx_track_get_total_time (GpxTrack* self) {
	time_t result;
	GpxPoint* a;
	GpxPoint* b;
	GList* na;
	GList* nb;
	gboolean _tmp0_;
	GpxPoint* _tmp2_;
	GpxPoint* _tmp1_;
	GpxPoint* _tmp4_;
	GpxPoint* _tmp3_;
	time_t time;
	a = NULL;
	b = NULL;
	na = g_list_first (self->points);
	nb = g_list_last (self->points);
	_tmp0_ = FALSE;
	if (na == NULL) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = nb == NULL;
	}
	if (_tmp0_) {
		result = 0;
		(a == NULL) ? NULL : (a = (gpx_point_unref (a), NULL));
		(b == NULL) ? NULL : (b = (gpx_point_unref (b), NULL));
		return result;
	}
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	a = (_tmp2_ = (_tmp1_ = (GpxPoint*) na->data, (_tmp1_ == NULL) ? NULL : gpx_point_ref (_tmp1_)), (a == NULL) ? NULL : (a = (gpx_point_unref (a), NULL)), _tmp2_);
	_tmp4_ = NULL;
	_tmp3_ = NULL;
	b = (_tmp4_ = (_tmp3_ = (GpxPoint*) nb->data, (_tmp3_ == NULL) ? NULL : gpx_point_ref (_tmp3_)), (b == NULL) ? NULL : (b = (gpx_point_unref (b), NULL)), _tmp4_);
	time = gpx_point_get_time (b) - gpx_point_get_time (a);
	result = time;
	(a == NULL) ? NULL : (a = (gpx_point_unref (a), NULL));
	(b == NULL) ? NULL : (b = (gpx_point_unref (b), NULL));
	return result;
}


/**
 * Try not to calculate time that we "stopped"  in average
 */
double gpx_track_calculate_moving_average (GpxTrack* self, GpxPoint* start, GpxPoint* stop, time_t* moving_time) {
	double result;
	double time;
	double distance;
	GList* iter;
	g_return_val_if_fail (self != NULL, 0.0);
	g_return_val_if_fail (start != NULL, 0.0);
	g_return_val_if_fail (stop != NULL, 0.0);
	time = (double) 0;
	distance = (double) 0;
	(*moving_time) = 0;
	iter = g_list_find (self->points, start);
	if (iter == NULL) {
		result = (double) 0;
		return result;
	}
	if (iter != NULL) {
		while (TRUE) {
			gboolean _tmp0_;
			GpxPoint* _tmp1_;
			GpxPoint* b;
			_tmp0_ = FALSE;
			if ((iter = iter->next) != NULL) {
				_tmp0_ = ((GpxPoint*) iter->prev->data) != stop;
			} else {
				_tmp0_ = FALSE;
			}
			if (!_tmp0_) {
				break;
			}
			_tmp1_ = NULL;
			b = (_tmp1_ = (GpxPoint*) iter->data, (_tmp1_ == NULL) ? NULL : gpx_point_ref (_tmp1_));
			if ((((b->distance - ((GpxPoint*) iter->prev->data)->distance) * 3600) / (gpx_point_get_time (b) - gpx_point_get_time ((GpxPoint*) iter->prev->data))) > 1.0) {
				time = time + ((double) (gpx_point_get_time (b) - gpx_point_get_time ((GpxPoint*) iter->prev->data)));
				distance = distance + (b->distance - ((GpxPoint*) iter->prev->data)->distance);
			}
			(b == NULL) ? NULL : (b = (gpx_point_unref (b), NULL));
		}
	}
	(*moving_time) = (time_t) time;
	result = distance / (time / (60.0 * 60.0));
	return result;
}


/* Calculate distance between point a and point b using great circular distance method 
 * Elevation is not taken into account.
 */
static double gpx_track_calculate_distance (GpxTrack* self, GpxPoint* a, GpxPoint* b) {
	double result;
	double retv;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, 0.0);
	g_return_val_if_fail (a != NULL, 0.0);
	g_return_val_if_fail (b != NULL, 0.0);
	retv = (double) 0;
	_tmp0_ = FALSE;
	if (a->lat == b->lat) {
		_tmp0_ = a->lon == b->lon;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = (double) 0;
		return result;
	}
	retv = 6378.7 * acos ((sin (a->lat) * sin (b->lat)) + ((cos (a->lat) * cos (b->lat)) * cos (b->lon - a->lon)));
	if (isnan (retv) == 1) {
		result = (double) 0;
		return result;
	}
	result = retv;
	return result;
}


/**
 * This class represents a Track in a gpx file.
 * The tracks contains the points connecting everything together. 
 * Info like total distance, average speed, moving speed/time etc are available.
 */
GpxTrack* gpx_track_construct (GType object_type) {
	GpxTrack * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


GpxTrack* gpx_track_new (void) {
	return gpx_track_construct (GPX_TYPE_TRACK);
}


const char* gpx_track_get_name (GpxTrack* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_name;
	return result;
}


void gpx_track_set_name (GpxTrack* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_name = (g_free (self->priv->_name), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "name");
}


static void gpx_track_class_init (GpxTrackClass * klass) {
	gpx_track_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GpxTrackPrivate));
	G_OBJECT_CLASS (klass)->get_property = gpx_track_get_property;
	G_OBJECT_CLASS (klass)->set_property = gpx_track_set_property;
	G_OBJECT_CLASS (klass)->finalize = gpx_track_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GPX_TRACK_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void gpx_track_instance_init (GpxTrack * self) {
	self->priv = GPX_TRACK_GET_PRIVATE (self);
	self->priv->_name = NULL;
	self->total_distance = 0.0;
	self->max_speed = (double) 0;
	self->points = NULL;
	self->priv->last = NULL;
	self->top = NULL;
	self->bottom = NULL;
}


static void gpx_track_finalize (GObject* obj) {
	GpxTrack * self;
	self = GPX_TRACK (obj);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	(self->points == NULL) ? NULL : (self->points = (_g_list_free_gpx_point_unref (self->points), NULL));
	(self->priv->last == NULL) ? NULL : (self->priv->last = (gpx_point_unref (self->priv->last), NULL));
	(self->top == NULL) ? NULL : (self->top = (gpx_point_unref (self->top), NULL));
	(self->bottom == NULL) ? NULL : (self->bottom = (gpx_point_unref (self->bottom), NULL));
	G_OBJECT_CLASS (gpx_track_parent_class)->finalize (obj);
}


GType gpx_track_get_type (void) {
	static GType gpx_track_type_id = 0;
	if (gpx_track_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GpxTrackClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gpx_track_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GpxTrack), 0, (GInstanceInitFunc) gpx_track_instance_init, NULL };
		gpx_track_type_id = g_type_register_static (G_TYPE_OBJECT, "GpxTrack", &g_define_type_info, 0);
	}
	return gpx_track_type_id;
}


static void gpx_track_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GpxTrack * self;
	gpointer boxed;
	self = GPX_TRACK (object);
	switch (property_id) {
		case GPX_TRACK_NAME:
		g_value_set_string (value, gpx_track_get_name (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gpx_track_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GpxTrack * self;
	self = GPX_TRACK (object);
	switch (property_id) {
		case GPX_TRACK_NAME:
		gpx_track_set_name (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


static void gpx_file_parse_track (GpxFile* self, xmlNode* node) {
	GpxTrack* track;
	xmlNode* trkseg;
	GpxTrack* _tmp3_;
	g_return_if_fail (self != NULL);
	/* Create new track here */
	track = gpx_track_new ();
	trkseg = node->children;
	while (TRUE) {
		if (!(trkseg != NULL)) {
			break;
		}
		if (_vala_strcmp0 (trkseg->name, "trkseg") == 0) {
			xmlNode* point;
			point = trkseg->children;
			while (TRUE) {
				if (!(point != NULL)) {
					break;
				}
				if (_vala_strcmp0 (point->name, "trkpt") == 0) {
					char* lat;
					char* lon;
					gboolean _tmp0_;
					lat = xmlGetProp (point, "lat");
					lon = xmlGetProp (point, "lon");
					_tmp0_ = FALSE;
					if (lat != NULL) {
						_tmp0_ = lon != NULL;
					} else {
						_tmp0_ = FALSE;
					}
					if (_tmp0_) {
						GpxPoint* p;
						double flat;
						double flon;
						xmlNode* info;
						p = gpx_point_new ();
						flat = g_ascii_strtod (lat, NULL);
						flon = g_ascii_strtod (lon, NULL);
						gpx_point_set_position (p, flat, flon);
						info = point->children;
						while (TRUE) {
							if (!(info != NULL)) {
								break;
							}
							/* height */
							if (_vala_strcmp0 (info->name, "ele") == 0) {
								char* content;
								content = xmlNodeGetContent (info);
								if (content != NULL) {
									p->elevation = g_ascii_strtod (content, NULL);
								}
								content = (g_free (content), NULL);
							} else {
								if (_vala_strcmp0 (info->name, "time") == 0) {
									char* _tmp1_;
									_tmp1_ = NULL;
									p->time = (_tmp1_ = xmlNodeGetContent (info), p->time = (g_free (p->time), NULL), _tmp1_);
								}
							}
							info = info->next;
						}
						gpx_track_add_point (track, p);
						(p == NULL) ? NULL : (p = (gpx_point_unref (p), NULL));
					} else {
						g_message ("gpx-parser.vala:267: Failed to get point: %s\n", point->name);
					}
					lat = (g_free (lat), NULL);
					lon = (g_free (lon), NULL);
				}
				point = point->next;
			}
		}
		if (_vala_strcmp0 (trkseg->name, "name") == 0) {
			if (gpx_track_get_name (track) == NULL) {
				char* _tmp2_;
				_tmp2_ = NULL;
				gpx_track_set_name (track, _tmp2_ = xmlNodeGetContent (trkseg));
				_tmp2_ = (g_free (_tmp2_), NULL);
			} else {
				g_warning ("gpx-parser.vala:278: Track name allready set: %s\n", gpx_track_get_name (track));
			}
		}
		trkseg = trkseg->next;
	}
	_tmp3_ = NULL;
	self->tracks = g_list_append (self->tracks, (_tmp3_ = track, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)));
	(track == NULL) ? NULL : (track = (g_object_unref (track), NULL));
}


static void gpx_file_parse_waypoint (GpxFile* self, xmlNode* node) {
	char* lat;
	char* lon;
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	lat = xmlGetProp (node, "lat");
	lon = xmlGetProp (node, "lon");
	_tmp0_ = FALSE;
	if (lat != NULL) {
		_tmp0_ = lon != NULL;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		GpxPoint* p;
		double flat;
		double flon;
		xmlNode* info;
		GpxPoint* _tmp2_;
		p = gpx_point_new ();
		flat = g_ascii_strtod (lat, NULL);
		flon = g_ascii_strtod (lon, NULL);
		gpx_point_set_position (p, flat, flon);
		info = node->children;
		while (TRUE) {
			if (!(info != NULL)) {
				break;
			}
			if (_vala_strcmp0 (info->name, "name") == 0) {
				if (gpx_point_get_name (p) == NULL) {
					char* _tmp1_;
					_tmp1_ = NULL;
					gpx_point_set_name (p, _tmp1_ = xmlNodeGetContent (info));
					_tmp1_ = (g_free (_tmp1_), NULL);
				} else {
					g_warning ("gpx-parser.vala:304: Point name allready set: %s\n", gpx_point_get_name (p));
				}
			}
			info = info->next;
		}
		_tmp2_ = NULL;
		self->waypoints = g_list_append (self->waypoints, (_tmp2_ = p, (_tmp2_ == NULL) ? NULL : gpx_point_ref (_tmp2_)));
		(p == NULL) ? NULL : (p = (gpx_point_unref (p), NULL));
	}
	lat = (g_free (lat), NULL);
	lon = (g_free (lon), NULL);
}


static void gpx_file_parse_route (GpxFile* self, xmlNode* node) {
	GpxTrack* track;
	xmlNode* trkseg;
	GpxTrack* _tmp3_;
	g_return_if_fail (self != NULL);
	/* Create new track here */
	track = gpx_track_new ();
	trkseg = node->children;
	while (TRUE) {
		if (!(trkseg != NULL)) {
			break;
		}
		if (_vala_strcmp0 (trkseg->name, "rtept") == 0) {
			char* lat;
			char* lon;
			gboolean _tmp0_;
			lat = xmlGetProp (trkseg, "lat");
			lon = xmlGetProp (trkseg, "lon");
			_tmp0_ = FALSE;
			if (lat != NULL) {
				_tmp0_ = lon != NULL;
			} else {
				_tmp0_ = FALSE;
			}
			if (_tmp0_) {
				GpxPoint* p;
				double flat;
				double flon;
				xmlNode* info;
				p = gpx_point_new ();
				flat = g_ascii_strtod (lat, NULL);
				flon = g_ascii_strtod (lon, NULL);
				gpx_point_set_position (p, flat, flon);
				info = trkseg->children;
				while (TRUE) {
					if (!(info != NULL)) {
						break;
					}
					/* height */
					if (_vala_strcmp0 (info->name, "ele") == 0) {
						char* content;
						content = xmlNodeGetContent (info);
						if (content != NULL) {
							p->elevation = g_ascii_strtod (content, NULL);
						}
						content = (g_free (content), NULL);
					} else {
						if (_vala_strcmp0 (info->name, "time") == 0) {
							char* _tmp1_;
							_tmp1_ = NULL;
							p->time = (_tmp1_ = xmlNodeGetContent (info), p->time = (g_free (p->time), NULL), _tmp1_);
						}
					}
					info = info->next;
				}
				gpx_track_add_point (track, p);
				(p == NULL) ? NULL : (p = (gpx_point_unref (p), NULL));
			} else {
				g_message ("gpx-parser.vala:345: Failed to get trkseg: %s\n", trkseg->name);
			}
			lat = (g_free (lat), NULL);
			lon = (g_free (lon), NULL);
		} else {
			if (_vala_strcmp0 (trkseg->name, "name") == 0) {
				if (gpx_track_get_name (track) == NULL) {
					char* _tmp2_;
					_tmp2_ = NULL;
					gpx_track_set_name (track, _tmp2_ = xmlNodeGetContent (trkseg));
					_tmp2_ = (g_free (_tmp2_), NULL);
				} else {
					g_warning ("gpx-parser.vala:351: Track name allready set: %s\n", gpx_track_get_name (track));
				}
			}
		}
		trkseg = trkseg->next;
	}
	_tmp3_ = NULL;
	self->routes = g_list_append (self->routes, (_tmp3_ = track, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)));
	(track == NULL) ? NULL : (track = (g_object_unref (track), NULL));
}


static gint gpx_file_read_file (GpxFile* self, gchar* buffer, int buffer_length1) {
	gint result;
	GError * _inner_error_;
	g_return_val_if_fail (self != NULL, 0);
	_inner_error_ = NULL;
	{
		gssize value;
		value = g_input_stream_read ((GInputStream*) self->priv->stream, buffer, (gsize) buffer_length1, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch0_g_error;
			goto __finally0;
		}
		result = (gint) value;
		return result;
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_critical ("gpx-parser.vala:372: error reading from stream: %s\n", e->message);
			result = -1;
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			return result;
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
}


static gint gpx_file_close_file (GpxFile* self) {
	gint result;
	GFileInputStream* _tmp0_;
	g_return_val_if_fail (self != NULL, 0);
	g_log ("GPX PARSER", G_LOG_LEVEL_DEBUG, "gpx-parser.vala:378: Close_file()");
	_tmp0_ = NULL;
	self->priv->stream = (_tmp0_ = NULL, (self->priv->stream == NULL) ? NULL : (self->priv->stream = (g_object_unref (self->priv->stream), NULL)), _tmp0_);
	result = 0;
	return result;
}


GpxFile* gpx_file_construct (GType object_type, const char* filename) {
	GError * _inner_error_;
	GpxFile * self;
	char* _tmp1_;
	const char* _tmp0_;
	GFile* _tmp2_;
	g_return_val_if_fail (filename != NULL, NULL);
	_inner_error_ = NULL;
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->filename = (_tmp1_ = (_tmp0_ = filename, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->filename = (g_free (self->filename), NULL), _tmp1_);
	_tmp2_ = NULL;
	self->priv->file = (_tmp2_ = g_file_new_for_commandline_arg (self->filename), (self->priv->file == NULL) ? NULL : (self->priv->file = (g_object_unref (self->priv->file), NULL)), _tmp2_);
	{
		GFileInputStream* _tmp3_;
		GFileInputStream* _tmp4_;
		xmlTextReader* reader;
		_tmp3_ = g_file_read (self->priv->file, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
		_tmp4_ = NULL;
		self->priv->stream = (_tmp4_ = _tmp3_, (self->priv->stream == NULL) ? NULL : (self->priv->stream = (g_object_unref (self->priv->stream), NULL)), _tmp4_);
		reader = xmlReaderForIO ((xmlInputReadCallback) gpx_file_read_file, (xmlInputCloseCallback) gpx_file_close_file, self, self->filename, "", 0);
		if (reader != NULL) {
			gint doc;
			/* Start parsing the xml file */
			doc = xmlTextReaderRead (reader);
			while (TRUE) {
				const char* _tmp5_;
				char* name;
				if (!(doc == 1)) {
					break;
				}
				_tmp5_ = NULL;
				name = (_tmp5_ = xmlTextReaderConstName (reader), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_));
				if (_vala_strcmp0 (name, "gpx") == 0) {
					gint doc2;
					doc2 = xmlTextReaderRead (reader);
					while (TRUE) {
						const char* _tmp6_;
						char* name2;
						if (!(doc2 == 1)) {
							break;
						}
						_tmp6_ = NULL;
						name2 = (_tmp6_ = xmlTextReaderConstName (reader), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_));
						/* Get the track element */
						if (_vala_strcmp0 (name2, "trk") == 0) {
							xmlNode* node;
							/* Track */
							node = xmlTextReaderExpand (reader);
							gpx_file_parse_track (self, node);
						} else {
							if (_vala_strcmp0 (name2, "wpt") == 0) {
								xmlNode* node;
								/* Waypoint */
								node = xmlTextReaderExpand (reader);
								gpx_file_parse_waypoint (self, node);
							} else {
								if (_vala_strcmp0 (name2, "rte") == 0) {
									xmlNode* node;
									node = xmlTextReaderExpand (reader);
									/* Route */
									gpx_file_parse_route (self, node);
								}
							}
						}
						doc2 = xmlTextReaderNext (reader);
						name2 = (g_free (name2), NULL);
					}
				} else {
					doc = xmlTextReaderRead (reader);
				}
				name = (g_free (name), NULL);
			}
		} else {
			/* Todo add error trower here http://www.vala-project.org/doc/vala-draft/errors.html#exceptionsexamples*/
			g_message ("gpx-parser.vala:428: Failed to open file");
		}
		xmlTextReaderClose (reader);
		(reader == NULL) ? NULL : (reader = (xmlFreeTextReader (reader), NULL));
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_critical ("gpx-parser.vala:432: failed to open file: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally1:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	return self;
}


GpxFile* gpx_file_new (const char* filename) {
	return gpx_file_construct (GPX_TYPE_FILE, filename);
}


static void gpx_file_class_init (GpxFileClass * klass) {
	gpx_file_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GpxFilePrivate));
	G_OBJECT_CLASS (klass)->finalize = gpx_file_finalize;
}


static void gpx_file_instance_init (GpxFile * self) {
	self->priv = GPX_FILE_GET_PRIVATE (self);
	self->filename = NULL;
	self->tracks = NULL;
	self->waypoints = NULL;
	self->routes = NULL;
	self->priv->file = NULL;
	self->priv->stream = NULL;
}


static void gpx_file_finalize (GObject* obj) {
	GpxFile * self;
	self = GPX_FILE (obj);
	self->filename = (g_free (self->filename), NULL);
	(self->tracks == NULL) ? NULL : (self->tracks = (_g_list_free_g_object_unref (self->tracks), NULL));
	(self->waypoints == NULL) ? NULL : (self->waypoints = (_g_list_free_gpx_point_unref (self->waypoints), NULL));
	(self->routes == NULL) ? NULL : (self->routes = (_g_list_free_g_object_unref (self->routes), NULL));
	(self->priv->file == NULL) ? NULL : (self->priv->file = (g_object_unref (self->priv->file), NULL));
	(self->priv->stream == NULL) ? NULL : (self->priv->stream = (g_object_unref (self->priv->stream), NULL));
	G_OBJECT_CLASS (gpx_file_parent_class)->finalize (obj);
}


GType gpx_file_get_type (void) {
	static GType gpx_file_type_id = 0;
	if (gpx_file_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GpxFileClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gpx_file_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GpxFile), 0, (GInstanceInitFunc) gpx_file_instance_init, NULL };
		gpx_file_type_id = g_type_register_static (G_TYPE_OBJECT, "GpxFile", &g_define_type_info, 0);
	}
	return gpx_file_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




